#$Id: Lobjectref.pm 97 2007-06-17 13:18:56Z zag $

package HTML::WebDAO::Lexer::Lobjectref;
use HTML::WebDAO::Lexer::Lbase;
use Data::Dumper;
use base qw( HTML::WebDAO::Lexer::Lregclass );
use strict;
1;
